part of pdf;

/// Represents the appearance of an annotation's border.
/// [PdfAnnotationBorder] class is used to create the annotation border
class PdfAnnotationBorder implements _IPdfWrapper {
  //constructor
  /// Initializes a new instance of the [PdfAnnotationBorder] class with specified border width, horizontal and vertical radius.
  PdfAnnotationBorder(
      [double borderWidth, double horizontalRadius, double verticalRadius]) {
    _toIntialize(borderWidth, horizontalRadius, verticalRadius);
  }

  // fields
  double _horizontalRadius = 0;
  double _verticalRadius = 0;
  double _borderWidth = 1;
  final _PdfArray _array = _PdfArray();

  // properties
  /// Gets a horizontal corner radius of the annotations.
  double get horizontalRadius => _horizontalRadius;

  /// Sets a horizontal corner radius of the annotations.
  set horizontalRadius(double value) {
    if (value != _horizontalRadius && value != null) {
      _horizontalRadius = value;
      _setNumber(0, value);
    }
  }

  /// Gets a vertical corner radius of the annotation.
  double get verticalRadius => _verticalRadius;

  /// Sets a vertical corner radius of the annotation.
  set verticalRadius(double value) {
    if (value != _verticalRadius && value != null) {
      _verticalRadius = value;
      _setNumber(1, value);
    }
  }

  /// Gets the width of annotation's border.
  double get width => _borderWidth;

  /// Sets the width of annotation's border.
  set width(double value) {
    if (value != _borderWidth && value != null) {
      _borderWidth = value;
    }
    _setNumber(2, value);
  }

  // implementation
  void _toIntialize(
      double borderWidth, double horizontalRadius, double verticalRadius) {
    if (borderWidth != null &&
        (horizontalRadius == null && verticalRadius == null)) {
      _initialize(borderWidth, this.horizontalRadius, this.verticalRadius);
      width = borderWidth;
    } else if (borderWidth == null &&
        (horizontalRadius == null && verticalRadius == null)) {
      _initialize(width, this.horizontalRadius, this.verticalRadius);
    } else {
      _initialize(borderWidth, horizontalRadius, verticalRadius);
      width = borderWidth;
      this.horizontalRadius = horizontalRadius;
      this.verticalRadius = verticalRadius;
    }
  }

  void _initialize(
      double borderWidth, double horizontalRadius, double verticalRadius) {
    _array._add(_PdfNumber(horizontalRadius));
    _array._add(_PdfNumber(verticalRadius));
    _array._add(_PdfNumber(borderWidth));
    _element = _array;
  }

  void _setNumber(int index, double value) {
    final _PdfNumber number = _array[index] as _PdfNumber;
    number.value = value;
  }

  @override
  _IPdfPrimitive _element;
}
