part of pdf;

/// Represents the class for text web link annotation.
class PdfTextWebLink {
  // Constructor
  /// Initializes a new instance of the [PdfTextWebLink] class.
  PdfTextWebLink(
      {@required String url,
      @required String text,
      PdfBrush brush,
      PdfFont font,
      PdfPen pen,
      PdfStringFormat format}) {
    _initialize(text, font, pen, brush, format);
    this.url = url;
  }

  // fields
  String _url;
  PdfUriAnnotation _uriAnnotation;

  /// Get or sets the text.
  String text;

  /// Get or sets the font.
  PdfFont font;

  /// Get or sets the pen.
  PdfPen pen;

  /// Get or sets the brush.
  PdfBrush brush;

  /// Get or sets the stringFormat.
  PdfStringFormat stringFormat;

  // properties
  /// Gets the Uri address.
  String get url => _url;

  /// Sets the Uri address.
  set url(String value) {
    ArgumentError.checkNotNull(value, 'url');
    if (value == '') {
      throw ArgumentError.value('Url - string can not be empty');
    }
    _url = value;
  }

  // implementation
  void _initialize(String text, PdfFont font, PdfPen pen, PdfBrush brush,
      PdfStringFormat format) {
    this.text = text != null && text.isNotEmpty ? text : '';
    if (font != null) {
      this.font = font;
    }
    if (brush != null) {
      this.brush = brush;
    }
    if (pen != null) {
      this.pen = pen;
    }
    if (format != null) {
      stringFormat = format;
    }
  }

  /// Draws a Text Web Link on the Page or graphics.
  void draw(PdfPage page, Offset location) {
    final Size textSize = font.measureString(text);
    final Rect rect = Rect.fromLTWH(
        location.dx, location.dy, textSize.width, textSize.height);
    _uriAnnotation = PdfUriAnnotation(bounds: rect, uri: url);
    _uriAnnotation.border = PdfAnnotationBorder(0, 0, 0);
    page.annotations.add(_uriAnnotation);
    _drawInternal(page.graphics, rect);
  }

  void _drawInternal(PdfGraphics graphics, Rect bounds) {
    ArgumentError.checkNotNull(graphics);
    ArgumentError.checkNotNull(font, 'Font cannot be null');
    graphics.drawString(text, font,
        pen: pen, brush: brush, bounds: bounds, format: stringFormat);
  }
}
