part of pdf;

/// Represents an named destination which goes to a destination in the current document.
class PdfNamedDestination implements _IPdfWrapper {
  /// Initializes a new instance of the [PdfNamedDestination] class
  /// with the title to be displayed.
  PdfNamedDestination(String title) {
    ArgumentError.checkNotNull(title, 'The title can\'t be null');
    this.title = title;
    _initialize();
  }

  /// Internal variable to store named destination's destination.
  PdfDestination _destination;

  /// Internal variable to store dictinary.
  final _PdfDictionary _dictionary = _PdfDictionary();

  /// Gets the named destination's destination.
  PdfDestination get destination => _destination;

  /// Sets the named destination's destination.
  /// The destination property has to be mentioned as multiples of 100.
  /// If we mention as 2, the zoom value will be 200.
  set destination(PdfDestination value) {
    ArgumentError.checkNotNull(value, 'The title can\'t be null');
    if (value != null) {
      _destination = value;
      _dictionary.setProperty(_DictionaryProperties.d, _destination);
    }
  }

  /// Gets the named destination title.
  String get title {
    final _PdfString title =
        _dictionary[_DictionaryProperties.title] as _PdfString;
    String value;
    if (title != null) {
      value = title.value;
    }
    return value;
  }

  /// Sets the named destination title.
  set title(String value) {
    ArgumentError.checkNotNull(value, 'The title can\'t be null');
    _dictionary[_DictionaryProperties.title] = _PdfString(value);
  }

  /// Initializes instance.
  void _initialize() {
    _dictionary._beginSave = (Object sender, _SavePdfPrimitiveArgs ars) {
      _dictionary.setProperty(_DictionaryProperties.d, _destination);
    };
    _dictionary.setProperty(
        _DictionaryProperties.s, _PdfName(_DictionaryProperties.goTo));
  }

  /// Gets the element.
  @override
  _IPdfPrimitive get _element => _dictionary;

  set _element(_IPdfPrimitive value) {
    throw ArgumentError();
  }
}
