part of pdf;

class _PdfArray implements _IPdfPrimitive, _IPdfChangable {
  _PdfArray([dynamic data]) {
    _elements ??= <_IPdfPrimitive>[];
    if (data != null) {
      if (data is _PdfArray) {
        _elements.addAll(data._elements);
      } else if (data is List<num>) {
        for (num entry in data) {
          final _PdfNumber pdfNumber = _PdfNumber(entry);
          _elements.add(pdfNumber);
        }
      } else if (data is List<_PdfArray>) {
        data.forEach(_elements.add);
      }
    }
  }

  //Constants
  static const String startMark = '[';
  static const String endMark = ']';

  //Fields
  List<_IPdfPrimitive> _elements;
  bool _isChanged;
  bool _isSaving;
  int _objectCollectionIndex;
  int _position;
  _ObjectStatus _status;

  //Properties
  _IPdfPrimitive operator [](int index) => _getElement(index);
  _IPdfPrimitive _getElement(int index) {
    return _elements[index];
  }

  void _add(_IPdfPrimitive element) {
    ArgumentError.checkNotNull(element, 'element');
    _elements.add(element);
  }

  bool _contains(_IPdfPrimitive element) {
    ArgumentError.checkNotNull(element, 'element');
    return _elements.contains(element);
  }

  void _insert(int index, _IPdfPrimitive element) {
    ArgumentError.checkNotNull(element, 'element');
    if (index >= _elements.length) {
      throw ArgumentError.value('index out of range $index');
    } else {
      _elements.insert(index, element);
    }
  }

  /// Cleares the array.
  void _clear() {
    _elements.clear();
    _isChanged = true;
  }

  int _indexOf(_IPdfPrimitive element) {
    ArgumentError.checkNotNull(element, 'element');
    return _elements.indexOf(element);
  }

  int get count => _elements.length;

  //Static methods
  static _PdfArray fromRectangle(_Rectangle rectanlge) {
    final List<double> list = <double>[
      rectanlge.left,
      rectanlge.top,
      rectanlge.right,
      rectanlge.bottom
    ];
    return _PdfArray(list);
  }

  //_IPdfPrimitive members
  @override
  bool get changed {
    _isChanged ??= false;
    return _isChanged;
  }

  @override
  set changed(bool value) {
    _isChanged = value;
  }

  @override
  _IPdfPrimitive clonedObject;
  @override
  bool get isSaving {
    _isSaving ??= false;
    return _isSaving;
  }

  @override
  set isSaving(bool value) {
    _isSaving = value;
  }

  @override
  int get objectCollectionIndex {
    _objectCollectionIndex ??= 0;
    return _objectCollectionIndex;
  }

  @override
  set objectCollectionIndex(int value) {
    _objectCollectionIndex = value;
  }

  @override
  int get position {
    _position ??= -1;
    return _position;
  }

  @override
  set position(int value) {
    _position = value;
  }

  @override
  _ObjectStatus get status {
    _status ??= _ObjectStatus.none;
    return _status;
  }

  @override
  set status(_ObjectStatus value) {
    _status = value;
  }

  @override
  void save(_IPdfWriter writer) {
    ArgumentError.checkNotNull(writer, 'writer');
    writer._write(startMark);
    for (int i = 0; i < count; i++) {
      this[i].save(writer);
      if (i + 1 != count) {
        writer._write(_Operators.whiteSpace);
      }
    }
    writer._write(endMark);
  }

  @override
  void dispose() {
    if (_elements != null) {
      _elements.clear();
      _elements = null;
    }
    if (_status != null) {
      _status = null;
    }
  }

  //_IPdfChangable members
  @override
  void freezeChanges(Object freezer) {}
}
